import logging
from mod_python import apache

try:
    import threading
    _current_thread = threading.currentThread
except:
    def _current_thread():
        return None

_cache = {}

def _discard_request(thread):
    try:
        _cache[thread].pop()
        if len(_cache[thread]) == 0:
            del _cache[thread]
    except:
        pass

def _cache_request(req):
    thread = _current_thread()
    if _cache.has_key(thread):
        if _cache[thread][-1] == req:
            return
        _cache[thread].append(req)
    else:
        _cache[thread] = [req]

    req.register_cleanup(_discard_request,(thread,))

def current_request():
    try:
        thread = _current_thread()
        return _cache[thread][-1]
    except:
        pass

def handler(req):
    _cache_request(req)
    return apache.OK


class ApacheLogHandler(logging.Handler):
   """A handler class which sends all logging to Apache."""

   def __init__(self):
       logging.Handler.__init__(self)

       # This level mapping is possibly not adequate for
       # case where explicit log level is used as it may
       # fall between these values. Should do comparisons
       # of log level and shift down to greatest value
       # less than actual log level.

       self.level_mapping = { }

       self.level_mapping[logging.CRITICAL] = apache.APLOG_CRIT
       self.level_mapping[logging.ERROR] = apache.APLOG_ERR
       self.level_mapping[logging.WARNING] = apache.APLOG_WARNING
       self.level_mapping[logging.INFO] = apache.APLOG_INFO
       self.level_mapping[logging.DEBUG] = apache.APLOG_DEBUG

   def emit(self, record):
       """Emit a record."""
       msg = self.format(record)
       req = current_request()
       if req != None:
           req.log_error(msg,self.level_mapping[record.levelno])

       else:
           apache.log_error(msg)

# Want to make sure we only create handler once. If part
# of mod_python.apache that will be easy. In this bodgy
# example, because of module reloading it is harder.

if not globals().has_key("_apache_handler"):
    global _apache_handler
    _apache_handler = ApacheLogHandler()
    logger = logging.getLogger("mod_python")
    logger.addHandler(_apache_handler)
